/****************************************************************************
**
** Copyright (C) 2019 The Qt Company Ltd.
** Contact: https://www.qt.io/licensing/
**
** This file is part of the examples of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:BSD$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** BSD License Usage
** Alternatively, you may use this file under the terms of the BSD license
** as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of The Qt Company Ltd nor the names of its
**     contributors may be used to endorse or promote products derived
**     from this software without specific prior written permission.
**
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
**
** $QT_END_LICENSE$
**
****************************************************************************/

import QtQuick 2.11
import QtQuick.Window 2.11

import QtQuick3D 1.0

Window {
    visible: true
    width: 640
    height: 480
    title: qsTr("Picking Example")


    View3D {
        id: view
        anchors.fill: parent

        DirectionalLight {

        }

        PerspectiveCamera {
            id: cam
            z: -500
        }

        PerspectiveCamera {
            id: cam2
            z: -1000
        }

        Model {
            id: cube
            source: "#Cube"
            pickable: true
            property bool isTouched: false

            materials: DefaultMaterial {
                diffuseColor: cube.isTouched ? "green" : "blue"
            }
        }

        Model {
            id: cube2

            x: 200
            y: 100
            z: -10

            rotation.x: 45
            rotation.y: 45
            rotation.z: 45
            source: "#Cube"
            pickable: true
            property bool isTouched: false

            materials: DefaultMaterial {
                diffuseColor: cube2.isTouched ? "pink" : "red"
            }
        }
    }

    MouseArea {
        anchors.fill: view

        onClicked: {
            console.log("picking (" + mouse.x + ", " + mouse.y + ")")
            var result = view.pick(mouse.x, mouse.y);
            if (result) {
                console.log(result.objectHit);
                if (result.objectHit) {
                    result.objectHit.isTouched = !result.objectHit.isTouched;
                    console.log("The uvPosition : " + result.uvPosition);
                    console.log("The distance: " + result.distance);
                    console.log("The world coord. position: " + result.scenePosition);
                }
                console.log("The uvPosition : " + result.uvPosition);
                console.log("The distance: " + result.distance);
                console.log("The world coord. position: " + result.scenePosition);
            }
        }
    }
}
